require 'log_analysis/version'
require 'log_analysis/loading_data'
require 'log_analysis/user_identification'
require 'log_analysis/session_identification'
require 'log_analysis/transformation'
require 'log_analysis/data_mining'
require 'log_analysis/intepretation'
require 'time'

class LogAnalysis
  class Error < StandardError; end
  # Your code goes here...

  attr_accessor :path, :type, :match_uri, :conf, :sup, :origin_data

  def initialize(path, type = nil)
    @path         = path
    @type         = type
    @origin_data  = LoadingData.input(path, type)
  end

  def selecting_data
    return @origin_data if @match_uri.nil?

    @origin_data.select { |record| record.uri.match?(@match_uri) }
  end

  def preprocessing_data
    data = selecting_data
    return if data.nil? || data.empty?

    filter  = data.select { |record| record.status_200? && record.method_get? && record.uri_without_data && !record.robot? }
    user    = UserIdentification.execute(filter)
    session = SessionIdentification.execute(user)
    session
  end

  def transformation
    data = preprocessing_data
    Transformation.execute(data) unless data.nil? || data.empty?
  end

  def data_mining
    data    = transformation
    @conf ||= 0.5
    @sup  ||= 60
    DataMining.execute(data, @conf, @sup) unless data.nil? || data.empty?
  end

  def intepretation
    data = data_mining
    Intepretation.execute(data) unless data.nil? || data.empty?
  end
end
