require 'ipaddr'
require 'useragent'
require 'log_analysis/model/user_identity'
require 'active_support/core_ext/numeric/time'

class Record
  attr_accessor :time,
                :host,
                :status,
                :size,
                :request_length,
                :req,
                :method,
                :uri,
                :referer,
                :ua,
                :reqtime,
                :runtime,
                :apptime,
                :cache,
                :vhost,
                :user,
                :forwardedfor,
                :forwardedproto

  attr_reader   :params

  DATA_TYPE = %w[.txt .json .js .css .jpg .jpeg .gif .woff2 .ico .png .bmp .mp3 .wav .avi .mpeg .vmw .mpg .map .pdf .doc .svg .otf].freeze
  REGEX_BOT = /facebookexternalhit|Mediapartners-Google|AWS|-|Crawler|spider|Detection/.freeze

  def initialize(params)
    @params = params
    record_params.keys.each { |key| instance_variable_set("@#{key}", record_params[key]) }
  end

  def status_200?
    status == 200
  end

  def method_get?
    method == 'GET'
  end

  def uri_without_data
    return false if uri.nil? || uri.end_with?(*DATA_TYPE)

    true
  end

  def robot?
    ua.bot? || ua.to_s.match?(REGEX_BOT)
  end

  private

  def record_params
    {}.tap do |p|
      p['time']           = Time.parse(@params['time'].split.first.sub(/:/, ' ')) || nil
      p['host']           = IPAddr.new(@params['host'])
      p['status']         = @params['status'].to_i || 0
      p['size']           = @params['size'].to_i || 0
      p['request_length'] = @params['request_length'].to_i || 0
      p['req']            = validate_string(@params['req'])
      p['method']         = validate_string(@params['method'])
      p['uri']            = validate_string(@params['uri'])
      p['referer']        = validate_string(@params['referer'])
      p['ua']             = UserAgent.parse(@params['ua'])
      p['reqtime']        = @params['reqtime'].to_f
      p['runtime']        = @params['runtime'].to_f
      p['apptime']        = @params['apptime'].to_f
      p['cache']          = validate_string(@params['cache'])
      p['vhost']          = validate_string(@params['vhost'])
      p['user']           = @params['user'] || nil
      p['forwardedfor']   = validate_string(@params['forwardedfor'])
      p['forwardedproto'] = validate_string(@params['forwardedproto'])
    end
  rescue IPAddr::InvalidAddressError => e
    puts e
  end

  def validate_string(value)
    value == '-' ? nil : value
  end
end
